from typing import Dict


class Song:
    def __init__(self, lyrics: str, artist: str, yt_link: str):
        self._lyrics = lyrics
        self._artist = artist
        self._yt_link = yt_link

    @staticmethod
    def new(data: Dict):
        return Song(lyrics=data['lyrics'], artist=data['artist'], yt_link=data['yt_link'])

    @property
    def lyrics(self):
        return self._lyrics

    @property
    def artist(self):
        return self._artist

    @property
    def yt_link(self):
        return self._yt_link



import json
import os
import random
import subprocess
import pkg_resources
from typing import Dict
from git import Repo, Commit
from lintalong.song import Song


class LinterDoesNotExistException(Exception):
    def __init__(self, message, command: Dict):
        message = "Linting command \"{0}\" does not exist".format(" ".join(command))

        super().__init(message)


class NoChangedFilesException(Exception):
    pass


class LintAlong:
    def __init__(self, config: Dict, repo: Repo):
        self.config = config
        self.repo = repo

    def lint(self):
        try:
            with open(os.devnull, 'w') as FNULL:
                subprocess.run(self.config['linter'], stdout=FNULL, stderr=subprocess.STDOUT, check=True)
        except subprocess.CalledProcessError as error:
            if error.returncode == 2:
                raise LinterDoesNotExistException(command=self.config['linter'])
            raise error

    def stage_files(self):
        changed_files = self.repo.untracked_files
        changed_files += [item.a_path for item in self.repo.index.diff(None)]

        if len(changed_files) == 0:
            raise NoChangedFilesException

        for file in changed_files:
            self.repo.index.add([file])

    def commit_files(self) -> Commit:
        song = self._fetch_random_song()
        message = ":musical_note: {0}\r\n\r\nBy {1} [{2}]".format(song.lyrics, song.artist, song.yt_link)

        return self.repo.index.commit(message=message)

    def format_commit(self, commit: Commit) -> str:
        totals = commit.stats.total

        return "[{0} {1}] {2}\r\n{3} file(s) changed, {4} insertions(+), {5} deletions(-)".format(
            self.repo.active_branch,
            str(commit),
            commit.summary,
            totals['files'],
            totals['insertions'],
            totals['deletions']
        )

    def _fetch_random_song(self) -> Song:
        song = json.loads(pkg_resources.resource_string(__name__, 'lyrics_pool.json'))['pool']
        return Song.new(random.choice(song))



from __future__ import print_function, unicode_literals
from setuptools import setup, find_packages

from distutils.core import setup

__author__ = "fowbi"

with open("README.rst", 'r') as file:
    readme = file.read()

with open("LICENSE", 'r') as file:
    license = file.read()

setup(
    name='lint-along',
    version='0.0.2',
    packages=find_packages(),
    url='https://github.com/fowbi/lint-along',
    license=license,
    zip_safe=False,
    keywords='lint-along ',
    author='fowbi',
    author_email='dev@magier.com',
    description='lint-along',
    long_description=readme,
    entry_points={
        "console_scripts": [
            'lint-along = lintalong.cli:cli'
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import click
import os
import pkg_resources
import subprocess
import yaml
from typing import Dict
from pathlib import Path
from git import Repo
from lintalong.lint_along import LintAlong, NoChangedFilesException, LinterDoesNotExistException


@click.command()
def cli():
    lint_along = LintAlong(load_config(), Repo(os.getcwd()))

    try:
        lint_along.lint()
    except LinterDoesNotExistException as error:
        click.echo(error)
        return

    try:
        lint_along.stage_files()
    except NoChangedFilesException:
        click.echo("Nothing to lint along")
        return

    commit = lint_along.commit_files()

    click.echo(message=lint_along.format_commit(commit))


def load_config() -> Dict:
    config = yaml.safe_load(pkg_resources.resource_string(__name__, 'config.yml'))

    if Path("~/.lint-along.yml").is_file():
        with open("~/.lint-along.yml", "r") as handle:
            config.update(yaml.safe_load(handle.read()))

    if Path(".lint-along.yml").is_file():
        with open(".lint-along.yml", "r") as handle:
            config.update(yaml.safe_load(handle.read()))

    return config


if __name__ == '__main__':
    cli()
